sphm <- function(x,start,origin,end,strata,event,knots,nk=5,eps=1E-12,est,
                 dlike=0.000125,maxit=100,piter=F,time.inc,
                 haz0=F,offset=rep(0,length(event)),initial,norm=1,
                 quanseq,rp.st,rp.kn,wald)
                                                                  {
#z0(-3)
   cal <- match.call()
   haz0 <- as.logical(haz0)
   piter <- as.logical(piter) 
   parms <- double(16)
#z0(-3.25)
#-----------------------------------------------------------------
# set default values  
#-----------------------------------------------------------------
# z0(-2)
 if (missing(start)) start <- rep(0,length(event))
 if (missing(origin)) origin <- start
#z0(-1)
# ----------------------------------------------------------------
# define x matrix
#----------------------------------------------------------------- 
    time.units <- units(end)
#z0(0)  
    if(time.units=="")               {
    cat("Time variable in sphm has no units attribute, Assuming Year\n")
    time.units <- "Year"              }     
    if(missing(time.inc))                                        {
      if(time.units=="Year") time.inc<-30  else time.inc <- 1     }
#
  if(length(x)>0 & !is.matrix(x))    x <-as.matrix(x)
  xpres <- !is.null(x)   # length >0 and null(dim) handles
#                                # 0X0 strata matrix
#  
#
    if(xpres)                   {
        nrec <- nrow(x)
        ncov <- ncol(x)          }
       else                      {
        nrec <- length(event)
        ncov <- 0
        x <- matrix(0,ncol=1,nrow=nrec)
                                   }
#
#-----------------------------------------------------------------
# define strata
#-----------------------------------------------------------------
    if(is.null(strata))       {
      strata <- rep(1,nrec)
      num.strata <- 1               }   else           {
      strata <- strata
      num.strata <- max(unclass(strata))      }
#
    if (!xpres) nrec <- length(strata)
#-----------------------------------------------------------------
# No. of obs in event, start, end, origin,strata and offset should 
# have the same length     
#-----------------------------------------------------------------
      l <-  range(c(nrec,length(start),length(end),length(origin),
                         length(event),length(strata),length(offset)))         
    if (l[1]!=l[2]) 
       stop("No. of observations in event,end,start,origin  
                                   and strata must match")
#
#-----------------------------------------------------------------
# make the outcome table
#-----------------------------------------------------------------
   nna<- if(xpres)     
   !is.na(matxv(x,rep(0,ncol(x)))+start+end+origin+event+strata+offset)
           else         
        !is.na(start+end+origin+event+strata+offset)
    temp <- category(ifelse(nna,event,2), 0:2,
                     labels=c("Alive","Dead","Deleted"))
#
    if (num.strata==1) n.table <-table(temp)  else
                       n.table <-table(strata,temp)
#
#-----------------------------------------------------------------
#  check the event values
#-----------------------------------------------------------------
     if (!is.logical(event))        {
    if (any(event>1)) event<-event-1
    if(any(event!=0 & event!=1)) stop("Invalid event value")
         event <-  as.logical(event)            }
#------------------------------------------------------------------------
# define all non-missing values from input
#------------------------------------------------------------------------
#
  if(xpres)      x     <- x[nna,,drop=F]
  start <- start[nna] 
  origin  <- origin[nna]
  end   <- end[nna]
  strata<- strata[nna]
  event <- event[nna]
  offset <- offset[nna]
  nrec  <- length(event)
  nmiss <- sum(!nna)    
#
#-------------------------------------------------------------------------
# define knots
#------------------------------------------------------------------------
  if(missing(knots) & nk >2)                  {
     if(nk<3) stop("nk must be >=3")
     knots <- matrix(double(1),nrow=nk,ncol=num.strata)
#-------------------------------------------------------------------------
#   define unique quantile 
#-------------------------------------------------------------------------
#  
   uquant<-function(x,outer,nk,quanseq0)   {
    for(i in nk:100)           {
#       q<-unique(quantile(x[x>0],seq(outer,1.0-outer,length=i))) 
#       q<-unique(quantile(x,seq(outer,1.0-outer,length=i))) 
       q<-unique(quantile(x,quanseq0))
       if(length(q)==nk) return(q)  
       else stop(paste("Could not obtain",nk,"unique knots"))
                            }       
                                      }
      for(i in 1 : num.strata)      {
    xx <- end[event & strata==i]-origin[event & strata==i]
    n <- length(xx)
    if(n<6) stop(paste("fewer than 6 uncensored observations in stratum",i))
    outer <- .1
    if(nk>3) outer <- .05
    if(nk>6) outer <- .025
    knot <- uquant(xx,outer,nk,quanseq[,i])
    if(n<100)                  {
          xx <- sort(xx)
          knot[1] <- xx[5]
          knot[nk] <- xx[n-4]   }     
    knots[,i] <- sort(knot)
    if (knots[1,i]==0) { knots[1,i]<-0.05*knots[2,i]   }
    if(any(duplicated(knots[,i])))
    stop(paste("Could not form unique knots in stratum",i))
                                    }         }
#
#
#
 if(nk<3)                             {
 knots <- matrix(double(1),nrow=nk,ncol=num.strata)
 for(i in 1 : num.strata)    {
 if(nk==2) knot <-c(0,0.0001)
  else if(nk==1) knot <-c(0) 
  else stop(paste("no knots number being specified in stratum",i)) 
 knots[,i] <- sort(knot)
 if (knots[1,i]==0) { knots[1,i]<-0.05*knots[2,i]   }
 if(any(duplicated(knots[,i])))
 stop(paste("Could not form unique knots in stratum",i))                     
                             }        }
#
    if(rp.st==1|rp.st==2|rp.st==3)         {
      knots[,rp.st]<-rp.kn       
     if(wald) {knots[,rp.st+1]<-rp.kn }
                                           }
#
#
#-----------------------------------------------------------------
# define est
#------------------------------------------------------------------------
    if(!xpres)                      {
        xname <- NULL  
        if(!missing(est)) stop("est was given without x")
         x  <- 0
                                     }  else                         { 
       if(missing(est)) est <- 1:ncov   else if(!is.null(est))     {
          estr <- range(est)
          if(estr[1]<1 | estr[2]>ncov)
                       stop("est has illegal column number for x")
            if(any(duplicated(est)))stop("est has duplicates")  
                                                                  }
        xname <- dimnames(x) [[2]]
        xname1 <- dimnames(x) [[1]]
        if(is.null(xname)) xname <- paste("x[",1:ncov,"]",sep="")                                                                       }
   if (!xpres) est <-c(1)    # handle no x matrix
#
#
  if(xpres) nxin <- length(est)
  else nxin <- c(1)          # handle no x matrix 
#
#----------------------------------------------------------------------------
# find start,end origin and strata values
#--------------------------------------------------------------------------
    if (any(start <0)| any(end<0) | any(origin<0)| 
        any(strata<0))
        stop ("Start,end,origin and strata values must be >=0")
#
#-------------------------------------------------------------------------
# compute mean from the covariates
#------------------------------------------------------------------------
  if(xpres)  { 
  xmean <- double(ncov)
 for(i in 1:ncov) xmean[i] <- mean(x[,i])   }
  else xmean <- c(0)
#
#
#-----------------------------------------------------------------
# define parms
#----------------------------------------------------------------
    nc <- 0
    nparm <- nk*num.strata+nc
    ofpres <- !missing(offset)
    ofpres <- as.logical(ofpres)
    nrec <- as.integer(nrec)
    nparm <- as.integer(nparm)
    nk <- as.integer(nk)
    num.strata <- as.integer(num.strata)
    maxit <- as.integer(maxit)
    norm <- as.integer(norm)
    parms[1:12] <- c(nrec,nparm,nk,num.strata,as.integer(nc),eps,dlike,maxit,piter,ofpres,haz0,norm)
#
    dyn.load("/suserlib/mlmats.o")
    dyn.load("/users/yls/splus/sphm.new1.o")
#-----------------------------------------------------------------
#  set up initial values
#-----------------------------------------------------------------
     if(missing(initial))  
                {
     initial <- rep(0,nparm)    
     j <- 1
     for (i in 1:num.strata)        {
         s<- strata==i
         initial[j]<-sum(event[s])/sum(end[s]-start[s])
     j <-j+nk
                                    }    }
   if(length(initial)<nparm) initial <- c(initial,rep(0,nparm-length(initial)))
#
#  
   kint <- nk*num.strata
   maxtime <- max(end)
#
#
#-----------------------------------------------------------------
#  call ssphm.o
#-----------------------------------------------------------------
 storage.mode(x) <- "single"
# 
#parms has been defined in the beginning
  storage.mode(nxin) <-"integer"
  storage.mode(est) <-"integer"
  storage.mode(initial) <- "double"
  storage.mode(knots) <- "double" 
  storage.mode(offset) <- "single" 
  storage.mode(strata) <- "integer" 
  storage.mode(start) <- "single" 
  storage.mode(end) <- "single" 
  storage.mode(origin) <- "single" 
  storage.mode(maxtime) <- "single" 
  non.slopes<-nk*num.strata 
   h <- .Fortran("sphm",parms=parms,beta=initial,as.integer(0),0,x,offset,
              u=double(nparm),
              double(nparm*(nparm+1)/2), 
              vsq=double(nparm*nparm),mean=xmean,
              strata,start,end,
              origin,event,knots,
              double(nparm*3),
              pivot=integer(nparm),
              double(max(1,nc)))
#
   irank <- h$parms[14]
   loglik <- h$parms[15]
   if(irank<nparm)          {
            cat("singular information matrix in sphm 
               (rank=",irank,").", 
                 "Offending variables(s):\n")
            cat(paste(xname[est[h$pivot[nparm:(irank+1)]-nk*num.strata]],
              collapse=" "),"\n")
            return(list(fail=T)) 
                             }
#
   dvrg <- h$parms[13]>0
   if(dvrg) return(list(call=cal,fail=T))
   ncov <- as.integer(ncov)
   nc <- ncov
   nparm <- nk*num.strata+nc
#

   parms[c(2,5)] <- c(nparm,nc)
   initial <- h$beta
   if(length(initial)<nparm) initial <- c(initial,rep(0,nparm-length(initial)))
   storage.mode(initial) <- "double" 
#
#
   h <- .Fortran("sphm",parms=parms,beta=initial,nxin,est,x,offset,
              u=double(nparm),
              double(nparm*(nparm+1)/2), 
              vsq=double(nparm*nparm),mean=xmean,
              strata,start,end,
              origin,event,knots,   
              double(nparm*3),
              pivot=integer(nparm),
              double(max(1,nc)))
#
   irank <- h$parms[14]
   loglik <- c(loglik,h$parms[15])
   center <- h$parms[16]
#
#  
 if(irank<nparm)          {
            cat("singular information matrix in sphm 
               (rank=",irank,").", 
                 "Offending variables(s):\n")
            cat(paste(xname[est[h$pivot[nparm:(irank+1)]-nk*num.strata]],
              collapse=" "),"\n")
            return(list(fail=T)) 
                             }
#
   dvrg <- h$parms[13]>0
   if(dvrg) return(list(call=cal,fail=T))
#
   cov<-solvet(matrix(h$vsq,nrow=nparm),tol=1e-12)
if(T) {
# 
   namx <-dimnames(x)[[2]]
   namx <-xname
#
   if(num.strata==1) nams<-" "
   else nams <- dimnames(n.table)[[1]]
   nam.al <-paste("alpha",1:nk,sep=" ")
   nam<- NULL
#
   for(i in 1:num.strata) nam <- c(nam,paste(nams[i],":",nam.al,sep=" "))
   nam <- c(nam,namx)
   names(h$beta) <- nam
   names(h$u) <- nam
   dimnames(cov) <- list(nam,nam)  
}
#  
   rlist <- list(parms=h$parms,coef=h$beta,u=h$u,
                 var=cov,
                 knots=knots,pivot=h$pivot,fail=dvrg,
                 maxtime=maxtime,mean=h$mean,nxin=nxin,
                 time.units=time.units,time.inc=time.inc,loglik=loglik,
                 center=center,n=n.table,
                 non.slopes=non.slopes)      
#                  
    rlist$dimnames <- NULL
    rlist$dim <- NULL
    class(rlist) <- "sphm"
    rlist               
                                                                      }
