detach(2)
store()

#----------------------------------------------------------------------------
# New S code to run the weighted likelihood bootstrap on a Poisson regression
#----------------------------------------------------------------------------
#----------------------------------------------------------------------------
#
# Created by Michael Newton, Dept. of Statistics, U. of Wisconsin--Madison,
# (newton@stat.wisc.edu, 608-262-0086) on 12-9-92.  Please send comments or
# questions to the author.  This code may be used for non-commercial purposes
# and may be freely distributed.
#
#----------------------------------------------------------------------------
#
# Newton and Raftery (1992) (preprint and under consideration by the
# Research Section Committee of the Royal Statistical Society, 12-92;
# for an earlier draft, see Technical Report No. 199, Dept. of Stat.,
# U. of Washington, Seattle) introduce a procedure called the weighted  
# likelihood bootstrap (WLB)  for approximate Bayesian inference.  
# This file illustrates this procedure
# in a Poisson regression model. It is S code (1991 version or later).  
# See ``Statistical Models in S'', by J. M. Chambers and T. J. Hastie, 1992,
# (Wadsworth & Brooks/Cole) for information about S.
#
# The WLB is applied here to the ``solder.balance'' data in S.  Random
# weights are generated, and then a weighted Poisson likelihood is repeatedly
# maximized.  A density estimate of the resulting coefficients is computed
# and importance resampling weights are computed.  A sample is then
# drawn from the discrete distribution determined by these weights. 
# This sample is approximately a random sample from the 18-dimensional
# posterior under a flat prior.
#
#-------------------------------------------------------------------------
#-------------------------------------------------------------------------
 
# Make the data available.
 
#library(data)   NOT NEEDED

#FH: Don't fit this model - the fit is not used later 
# Fit the Poisson regression model.
 
#mod <- glm( skips ~ . ,  data=solder.balance, family=poisson )
 
# some useful dimensions 
 
nb <- 200  
#FH: Remobe the following 2 lines and instead compute n,p later
#n <- length(solder.balance$skips)
#p <- length(mod$coefficients)

# FH: Don't do the following - matrix will be too large to fit in
# memory for large problems, and it can be generated and used
# line by line instead
 
# Fill a matrix with nb columns of random weights.
# 
#alpha <- .7 
#wt <- ( matrix( -log(runif(n*nb)), n, nb ) )^alpha


# Prepare a streamlined  optimization routine.

#FH:In place of the following, generate the model matrix a little more elegantly
#xx <-  model.matrix(mod) 
xx <- model.matrix(skips~., data=solder.balance)
#FH:add the following 3 lines
dm <- dim(xx)
n <- dm[1]
p <- dm[2]

#FH: Note: This assumes no NAs present
y <- solder.balance$skips

#FH: Don't use the following function but compute as needed for a
#slight speed increase
#fit.wlb <- function(ww)
# {
#	mod2 <- glm.fit(xx,y,ww,family=poisson)
#	as.numeric( mod2$coefficients )
# }

# Get a batch of raw wlb samples.

coef.wlb <- matrix(NA,p,nb)
#FH:Replace the following line
#for( i in 1:nb ) { coef.wlb[,i] <- fit.wlb(wt[,i]) }
for( i in 1:nb ) coef.wlb[,i] <- glm.fit(xx, y, (-log(runif(n)))^.7,
  family=poisson)$coefficients
 
# If a larger sample is required, run the above code several times,
# or test the capacity of your version of S by simply increasing nb.

# To get importance weights, both the likelihood and the estimated density
# for each coef.wlb column must be computed.  

loglik <- function(theta)
	{
		eta <- xx %*% theta
		sum( y*( eta ) ) - sum( exp(eta) )
        }
loglik.wlb  <- apply(coef.wlb,2,loglik)

# Get a normal kernel density estimate. 
 
terrell <- ((p+8)^((p+6)/2)/(16*nb*(p+2)*gamma((p+8)/2)))^(2/(p+4))
vcv.wlb <- var(t(coef.wlb))
foo <- solve( chol(vcv.wlb*terrell) )
coef.t <- t(foo) %*% coef.wlb
ones <- matrix( rep(1,p), 1, p )
fhat <- rep(0,nb)
for( i in 1:nb)
 {
  tmp <- -.5*(coef.t[,i] - coef.t)^2
  blah <- ones %*% tmp 
  fhat[i] <- sum( exp(blah) )
#FH:Save space on output by replacing the following line
#  print(i)
  cat(i,'')
 }

# Compute importance weights.

iw <- loglik.wlb - log( fhat )
iw <- exp( iw - max(iw) )
iw <- iw/sum(iw)


# Finally, sample from the discrete distribution determined by these weights.

ind <- sample( (1:nb), replace=T, prob=iw )
coef.sir <- coef.wlb[,ind]
