/* ef_twist.c */
/* This program calculates the distribution of P(twist>x, where twist=
TRL-T1*, TRL=min(TR,L),T1*=min(TOX,TR), using the
more efficient estimator, and compares it with the true distribution and
the KM estimate. It also calculates the variance from our formula, and
compares it with the population variance, and the variance of the KM
estimator */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define ARRAYLEN 100
#define intval 20
#define step 4
#define theta 120
#define TOX2 72
#define FU1 48
#define FU2 96
#define numsim 2000
#define L  81

static double Power2(double x);
static void RandSim(double t1[], double trL[], double censor[]);
static void GenData(double t1[], double trL[], double x1[], int delta1[], 
	     double x2[], int delta2[], double censor[], int x);
static void CalCensor(int delta1[], int delta2[], double *percens1, 
		      double *percens2, double x2[], int *lastcens);
static void ChangeDelta(double time[], int delta[]);
static void ChangeKmSurv(double time[], double survprob[]);
static void CalKmSurv(double x[], double kmold[], int mytime[], 
       double kmsurv[]);
static double CalOurSurv(double x1[], double x2[], int delta2[], double kc[], 
		       int mytime);
static double CalModelLamda(double x[], int delta[]);
static double CalZup1(double lamda1, double lamda2, double x, double t1, 
		     double u);
static double CalZup2(double lamda1, double lamda2, double x, double t1, 
		     double u);
static double CalZup3(double lamda1, double lamda2, double x, double t1, 
		     double u);
static void CalSurvAdd(int mytime[], double x1[], double x2[], int delta2[], 
       double km[], double lamda1, double lamda2, double survadd[], 
       double varsub[], double CalZup (double lamda1, double lamda2, 
       double x, double t1, double u));
static double CalOurVar(int delta2[], double x1[], double x2[], double s[], 
	      double kc[], int mytime, double mysurv);

main()
{
  int     i, j, k, ii, mytime[intval], delta1[ARRAYLEN], delta2[ARRAYLEN];
  int    lastcens, c2[ARRAYLEN];
  double t1[ARRAYLEN], trL[ARRAYLEN], x1[ARRAYLEN], x2[ARRAYLEN],
         censor[ARRAYLEN], kmx[ARRAYLEN];
  double percens1=0., percens2=0., tppercens1, tppercens2, perlastcens=0., 
         lamdahat1, lamdahat2;
  double truesurv[intval], mysurv[intval], kmsurv[intval];
  double mynewsurv1[intval],mynewsurv2[intval],mynewsurv3[intval];
  double varmysurv[intval],varmynewsurv1[intval], varmynewsurv2[intval];
  double varmynewsurv3[intval];
  double varsub1[intval], varsub2[intval], varsub3[intval];
  double myvar[intval], tpmyvar[intval], mynewvar1[intval];
  double mynewvar2[intval], mynewvar3[intval];
  double stmynewvar[intval],  remysurv[intval];
  double tptruesurv[intval], tpmysurv[intval], tpkmsurv[intval];
  double survadd1[intval], survadd2[intval], survadd3[intval];
  double s[ARRAYLEN], kc[ARRAYLEN], kmold[ARRAYLEN];
  double x[ARRAYLEN], xmatx[2][ARRAYLEN], sprob[2][ARRAYLEN];
  FILE    *fp;

/* Open the output files */
        fp = fopen("outdata","w");

/* Set up the time frame */
  for (i=0; i<intval; i++) mytime[i]=step*(i+1);

/* Calculate the true distribution */
  for (i=0; i<intval; i++) {
    if (mytime[i]<L-TOX2)
      truesurv[i]=exp(-(double)mytime[i]/theta)*
                  (1-exp(-(double)TOX2/theta))*(double)theta/TOX2;
    else
      truesurv[i]=(exp(-(double)mytime[i]/theta)-
                   exp(-(double)L/theta))*(double)theta/TOX2;
  }

/* Initialization */
  for (i=0; i<intval; i++) {
    kmsurv[i]=0.;
    mysurv[i]=0.;
    varmysurv[i]=0.;
    myvar[i]=0.;
  }

for (k=0; k<numsim; k++){

/* Generate random variables */
  RandSim(t1, trL, censor);

/* Use KM method to calculate survival curve) */

  for (i=0; i<ARRAYLEN; i++) {
    x1[i]=t1[i];
    x2[i]=trL[i];
    delta1[i]=1;
    delta2[i]=1;
    if (t1[i]>censor[i]){
      delta1[i]=0;
      x1[i]=censor[i];
    }
    if (trL[i]>censor[i]){
      delta2[i]=0;
      x2[i]=censor[i];
    }
    kmx[i]=x2[i]-x1[i];
    xmatx[0][i]=kmx[i];
    xmatx[1][i]=1.0 - (double) delta2[i];
  }

  i=ARRAYLEN;
  km_(&i,xmatx,sprob);

  for(i=0;i<ARRAYLEN;i++) kmold[i]=sprob[0][i];

  ChangeKmSurv(kmx, kmold);
  CalKmSurv(kmx, kmold, mytime,tpkmsurv);

/* Using our new method */

 for (ii=0; ii<intval; ii++) { 

/* Generate sample data */
  GenData(t1, trL, x1, delta1, x2, delta2, censor, mytime[ii]);

/* Calculate percentage of observations that is censored 

  CalCensor(delta1,delta2,&tppercens1,&tppercens2,x2,&lastcens);

  perlastcens += (double)lastcens;
  percens1 += tppercens1;
  percens2 += tppercens2;
*/

/* Use our formula to calculate survival curve) */

  ChangeDelta(x2, delta2);

  /* First calculate the K-M estimator for the censoring distribution */

  for (i=0; i<ARRAYLEN; i++) {
    xmatx[0][i]=x2[i];
    xmatx[1][i]=(double)delta2[i];
  }

  i=ARRAYLEN;
  km_(&i,xmatx,sprob);  /* fortran subroutine, calls IMSL */
  for(i=0;i<ARRAYLEN;i++) kc[i]=sprob[0][i];
  ChangeKmSurv(x2, kc);

  /* Then use the formula to calculate our naive estimator */

  tpmysurv[ii]=CalOurSurv(x1, x2, delta2, kc, mytime[ii]);

/* Calculate the variance using our formula */

  /* First find out the K-M estimator for S(u)=pr(TR>u) */

  for (i=0; i<ARRAYLEN; i++) {
    xmatx[0][i]= x2[i];
    xmatx[1][i]= 1.0 - (double)delta2[i];
  }
  i=ARRAYLEN;
  km_(&i,xmatx,sprob);  /* fortran subroutine, calls IMSL */

  for(i=0;i<ARRAYLEN;i++) s[i]=sprob[0][i];
  ChangeKmSurv(x2, s);

  /* Then calculate the variance for our formula */

  tpmyvar[ii]=CalOurVar(delta2, x1, x2, s, kc, mytime[ii], tpmysurv[ii]);
}
if (k==0){
  for (i=0; i<intval; i++){
    kmsurv[i] = tpkmsurv[i];
    mysurv[i] = tpmysurv[i];
    myvar[i] = tpmyvar[i];
  }
}
else{
  for (i=0; i<intval; i++){
    kmsurv[i] += tpkmsurv[i];
    mysurv[i] += tpmysurv[i];
    varmysurv[i] += Power2(tpmysurv[i]-mysurv[i]/(k+1))*(k+1)/(k+2);
    myvar[i] += tpmyvar[i];
  }
}  
}
  for (i=0; i<intval; i++){
    kmsurv[i] /= numsim;
    mysurv[i] /= numsim;
    varmysurv[i] /= numsim-1;
    myvar[i] /= numsim;
  }
      
/* Output the data */

  for(i=0; i<intval; i++) {
     fprintf(fp, "%7d %7.4f %7.4f %7.4f %7.4f %7.4f \n",  
     mytime[i],truesurv[i],kmsurv[i],mysurv[i],varmysurv[i],myvar[i]);
   }
  close(fp);
}

/* Function to calculate the power of 2 */

double Power2(double x)
{
  double y;
  y=x*x;
  return(y);
}

/* Simulate T1, TR and Censor time */ 

void RandSim(double t1[], double trL[], double censor[])
{
  int i;
  double  unrand1[ARRAYLEN], unrand2[ARRAYLEN], tox[ARRAYLEN], 
          tr[ARRAYLEN], lamda;

  i = ARRAYLEN;
  lamda=theta;
  randep_(&i,&lamda,tr);  /* fortran sub routine, calls IMSL */
  randun_(&i,unrand1);  /* fortran sub routine, calls IMSL */
  randun_(&i,unrand2);  /* fortran sub routine, calls IMSL */

  for(i=0;i<ARRAYLEN;i++) {
    tox[i]=unrand1[i]*TOX2;
    censor[i]=unrand2[i]*(FU2-FU1)+FU1;

    trL[i]=tr[i];
    if(trL[i]>L) trL[i]=L;
    
    t1[i]=tox[i];
    if (t1[i]>trL[i]) t1[i]=trL[i];
  }
}
void GenData(double t1[], double trL[], double x1[], int delta1[], 
	     double x2[], int delta2[], double censor[], int x)
/* x1 is the censored t1, x2 is the censored tr
   delta1 is the indictor of t1, delta2 is the indicator for tr */
{
  int i;
  double  unrand1[ARRAYLEN], unrand2[ARRAYLEN], tox[ARRAYLEN], 
          tr[ARRAYLEN], lamda;

  i = ARRAYLEN;

  for(i=0;i<ARRAYLEN;i++) {
    x1[i]=t1[i];
    x2[i]=trL[i];
    delta1[i]=1;
    delta2[i]=1;
    if (t1[i]>censor[i]){
      delta1[i]=0;
      x1[i]=censor[i];
    }
    if (trL[i]>censor[i]){
      delta2[i]=0;
      x2[i]=censor[i];
    }
    if (x1[i]+x<x2[i]){
      delta2[i]=1;
      x2[i]=x1[i]+x+0.0001;
    }
  }
}
void CalCensor(int delta1[], int delta2[], double *percens1, double *percens2,
	       double x2[],  int *lastcens)
{
  int i, j;
  double temp=0.;
  *percens1=0.;
  *percens1=0.;
  for (i=0; i<ARRAYLEN; i++) {
    *percens1 += 1.0-(double)delta1[i];
    *percens2 += 1.0-(double)delta2[i];
    if (x2[i]>temp) {
      temp=x2[i];
      j=i;
    }
  }
  *lastcens=1-delta2[j];
  *percens1 /= ARRAYLEN;
  *percens2 /= ARRAYLEN;
}

void ChangeDelta(double time[], int delta[])
/* make the last obs a failure */
{
  double temp=0.;
  int i, j;
  for (i=0; i<ARRAYLEN; i++) {
    if (time[i]>temp) {
      temp=time[i];
      j=i;
    }
  }
  delta[j]=1;
}

void ChangeKmSurv(double time[], double surv[])
{
  int i, j;
  double temp=1000.;

  for (i=0; i<ARRAYLEN; i++) {
    if (surv[i]<temp) {
      temp=surv[i];
      j=i;
    }
  }
  for (i=0; i<ARRAYLEN; i++) {
    if (time[i] > time[j]) surv[i]=temp;
  }
}

void CalKmSurv(double x[], double kmold[], int mytime[], double kmsurv[])
/* get the K-M estimator at the selected points */
/* it is equal to the nearest point to the left */
{
  int i, j, k;
  double temp;
  for (i=0; i<intval; i++) {
    temp=1000.;
    k=1000;
    for (j=0; j<ARRAYLEN; j++){
      if((mytime[i]-x[j]<temp ) && (mytime[i]-x[j]>0)){
	temp=mytime[i]-x[j];
	k=j;
      }
    }
    if (k != 1000) kmsurv[i]=kmold[k];
    else kmsurv[i]=1.0;
  }
}

double CalOurSurv(double x1[], double x2[], int delta2[], double kc[], 
		int mytime)
{
  int i,j;
  double mysurv;
    mysurv=0.;
    for (j=0; j<ARRAYLEN; j++){
      if (delta2[j]==1 && x2[j]-x1[j]>=mytime) mysurv += 1.0/kc[j];
    }
    mysurv /= ARRAYLEN;
  return(mysurv);
}

double CalOurVar(int delta2[], double x1[], double x2[], double s[], 
		 double kc[], int mytime, double mysurv)
{
  int j, k;
  double e[ARRAYLEN], temp, myvar;

    myvar = 0.;
    for (j=0; j<ARRAYLEN; j++){
      e[j]=0.;
      if (delta2[j]==0){
	for (k=0; k<ARRAYLEN; k++){
	  if(delta2[k]==1 && x2[k]-x1[k]>=mytime && x2[k]>=x2[j]) 
	    e[j] += 1.0/kc[k];
	}
	e[j] = e[j]/(s[j]*ARRAYLEN);
	myvar += e[j]*(1.0-e[j])/(kc[j]*kc[j]);
      }
    }
    myvar = mysurv*(1.0-mysurv)+myvar/ARRAYLEN;
    myvar /= ARRAYLEN;
  return(myvar);
}

